/* ##############################################################################################
**
** Copyright 2012-2016 CNRS, INPT
** Copyright 2013-2015 UPS
**  
** This file is part of qr_mumps.
**  
** qr_mumps is free software: you can redistribute it and/or modify
** it under the terms of the GNU Lesser General Public License as 
** published by the Free Software Foundation, either version 3 of 
** the License, or (at your option) any later version.
**  
** qr_mumps is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Lesser General Public License for more details.
**  
** You can find a copy of the GNU Lesser General Public License
** in the qr_mumps/doc directory.
**
** ##############################################################################################*/


/*##############################################################################################*/
/** @file qrm_mumps.h
 * Header file for the C interface
 *
 * $Date: 2016-06-29 14:39:53 +0200 (Wed, 29 Jun 2016) $
 * $Author: abuttari $
 * $Version: 2.0 $
 * $Revision: 2244 $
 *
 */
/*##############################################################################################*/

#if defined(zprec) || defined(cprec)
#include <complex.h>
#endif

#include <string.h>
#include "qrm_common_c.h"

struct dqrm_spmat_type_c{
  int          *irn, *jcn;
  double  *val;
  int          m, n, nz;
  int          *cperm_in;
  int          icntl[20];
  double       rcntl[10];
  long int     gstats[10];
  int          h; 
};


int  dqrm_spmat_init_c(struct dqrm_spmat_type_c *qrm_spmat_c);
int  dqrm_spmat_destroy_c(struct dqrm_spmat_type_c *qrm_spmat_c);
void dqrm_readmat_c(char *matfile, struct dqrm_spmat_type_c *qrm_spmat_c);
int  dqrm_analyse_c(struct dqrm_spmat_type_c *qrm_spmat_c, const char transp);
int  dqrm_factorize_c(struct dqrm_spmat_type_c *qrm_spmat_c, const char transp);
int  dqrm_solve_c(struct dqrm_spmat_type_c *qrm_spmat_c, const char transp,
                  double *b, double *x, const int nrhs);
int  dqrm_apply_c(struct dqrm_spmat_type_c *qrm_spmat_c, const char transp,
                  double *b, const int nrhs);
void dqrm_matmul_c(struct dqrm_spmat_type_c *qrm_spmat_c, const char transp,
                   const double alpha, double *x, 
                   const double beta, double *y, 
                   const int nrhs);
int  dqrm_matnrm_c(struct dqrm_spmat_type_c *qrm_spmat_c, const char ntype, 
                   double *nrm);
int  dqrm_vecnrm_c(const double *x, const int n, const int nrhs, 
                   const char ntype, double *nrm);
int  dqrm_least_squares_c(struct dqrm_spmat_type_c *qrm_spmat_c, double *b, 
                          double *x, const int nrhs);
int  dqrm_min_norm_c(struct dqrm_spmat_type_c *qrm_spmat_c, double *b, 
                          double *x, const int nrhs);
int  dqrm_residual_norm_c(struct dqrm_spmat_type_c *qrm_spmat_c, double *b, 
                          double *x, const int nrhs, double *nrm);
int  dqrm_residual_orth_c(struct dqrm_spmat_type_c *qrm_spmat_c, double *r, 
                          const int nrhs, double *nrm);


int  dqrm_pseti_c(struct dqrm_spmat_type_c *qrm_spmat_c, const char *string, int val);
int  dqrm_pgeti_c(struct dqrm_spmat_type_c *qrm_spmat_c, const char *string, int *val);
int  dqrm_pgetii_c(struct dqrm_spmat_type_c *qrm_spmat_c, const char *string, long long *val);

