/* ##############################################################################################
**
** Copyright 2012-2016 CNRS, INPT
** Copyright 2013-2015 UPS
**  
** This file is part of qr_mumps.
**  
** qr_mumps is free software: you can redistribute it and/or modify
** it under the terms of the GNU Lesser General Public License as 
** published by the Free Software Foundation, either version 3 of 
** the License, or (at your option) any later version.
**  
** qr_mumps is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Lesser General Public License for more details.
**  
** You can find a copy of the GNU Lesser General Public License
** in the qr_mumps/doc directory.
**
** ##############################################################################################*/


/*##############################################################################################*/
/** @file qrm_mumps.h
 * Header file for the C interface
 *
 * $Date: 2016-06-29 14:39:53 +0200 (Wed, 29 Jun 2016) $
 * $Author: abuttari $
 * $Version: 2.0 $
 * $Revision: 2244 $
 *
 */
/*##############################################################################################*/

#if defined(zprec) || defined(cprec)
#include <complex.h>
#endif

#include <string.h>
#include "qrm_common_c.h"

struct sqrm_spmat_type_c{
  int          *irn, *jcn;
  float  *val;
  int          m, n, nz;
  int          *cperm_in;
  int          icntl[20];
  double       rcntl[10];
  long int     gstats[10];
  int          h; 
};


int  sqrm_spmat_init_c(struct sqrm_spmat_type_c *qrm_spmat_c);
int  sqrm_spmat_destroy_c(struct sqrm_spmat_type_c *qrm_spmat_c);
void sqrm_readmat_c(char *matfile, struct sqrm_spmat_type_c *qrm_spmat_c);
int  sqrm_analyse_c(struct sqrm_spmat_type_c *qrm_spmat_c, const char transp);
int  sqrm_factorize_c(struct sqrm_spmat_type_c *qrm_spmat_c, const char transp);
int  sqrm_solve_c(struct sqrm_spmat_type_c *qrm_spmat_c, const char transp,
                  float *b, float *x, const int nrhs);
int  sqrm_apply_c(struct sqrm_spmat_type_c *qrm_spmat_c, const char transp,
                  float *b, const int nrhs);
void sqrm_matmul_c(struct sqrm_spmat_type_c *qrm_spmat_c, const char transp,
                   const float alpha, float *x, 
                   const float beta, float *y, 
                   const int nrhs);
int  sqrm_matnrm_c(struct sqrm_spmat_type_c *qrm_spmat_c, const char ntype, 
                   float *nrm);
int  sqrm_vecnrm_c(const float *x, const int n, const int nrhs, 
                   const char ntype, float *nrm);
int  sqrm_least_squares_c(struct sqrm_spmat_type_c *qrm_spmat_c, float *b, 
                          float *x, const int nrhs);
int  sqrm_min_norm_c(struct sqrm_spmat_type_c *qrm_spmat_c, float *b, 
                          float *x, const int nrhs);
int  sqrm_residual_norm_c(struct sqrm_spmat_type_c *qrm_spmat_c, float *b, 
                          float *x, const int nrhs, float *nrm);
int  sqrm_residual_orth_c(struct sqrm_spmat_type_c *qrm_spmat_c, float *r, 
                          const int nrhs, float *nrm);


int  sqrm_pseti_c(struct sqrm_spmat_type_c *qrm_spmat_c, const char *string, int val);
int  sqrm_pgeti_c(struct sqrm_spmat_type_c *qrm_spmat_c, const char *string, int *val);
int  sqrm_pgetii_c(struct sqrm_spmat_type_c *qrm_spmat_c, const char *string, long long *val);

